@@ -199,6 +199,11 @@ configOptions::configOptions( QWidget * parent,
 
 		m_ui->lineEditHiDPI->setEnabled( e ) ;
 	} ) ;
+
+	connect( m_ui->chShowDebugWindowOnStartup,&QCheckBox::toggled,[ this ]( bool e ){
+
+		m_settings.showDebugWindowOnStartup( e ) ;
+	} ) ;
 }
 
 configOptions::~configOptions()
@@ -249,6 +254,8 @@ void configOptions::ShowUI()
 
 	m_ui->lineEditHiDPI->setEnabled( m_ui->cbHiDPI->isChecked() ) ;
 
+	m_ui->chShowDebugWindowOnStartup->setChecked( m_settings.showDebugWindowOnStartup() ) ;
+
 	if( utility::platformIsWindows() ){
 
 		m_ui->lineEditMountPointPrefix->setText( m_settings.windowsExecutableSearchPath() ) ;
