@@ -26,10 +26,6 @@ public:
 
 	engines::engine::status errorCode( const QString& e,int s ) const override ;
 
-	engines::engine::args command( const QByteArray& password,
-				       const engines::engine::cmdArgsList& args,
-				       bool create ) const override ;
-
 	bool requiresAPassword( const engines::engine::cmdArgsList& ) const override ;
 
 	void GUICreateOptions( const engines::engine::createGUIOptions& ) const override ;
