@@ -65,43 +65,11 @@
 #include "json.h"
 #include "settings.h"
 #include "engines.h"
+#include "processManager.h"
 
 #include "favorites2.h"
 #include "favorites.h"
 
-static keyDialog::volumeList _convert_lists( favorites::volumeList s,
-					     bool earlyBoot,
-					     bool skipUnknown )
-{
-	keyDialog::volumeList m ;
-
-	for( auto&& it : s ){
-
-		const auto& f = it.favorite() ;
-
-		engines::engineWithPaths e( f.volumePath,f.configFilePath ) ;
-
-		if( e->known() ){
-
-			m.emplace_back( std::move( it ),std::move( e ) ) ;
-
-		}else if( skipUnknown ){
-
-			utility::debug() << "Not Adding Not Available Volume: " + f.volumePath ;
-
-		}else if( !earlyBoot ){
-
-			utility::debug() << "Unknown Volume Type Detected: " + f.volumePath ;
-
-			m.emplace_back( std::move( it ),std::move( e ) ) ;
-		}else{
-			utility::debug() << "Skipping Not Available Volume: " + f.volumePath ;
-		}
-	}
-
-	return m ;
-}
-
 static std::function< void( const QString& ) > _debug()
 {
 	return []( const QString& e ){ utility::debug() << e ; } ;
@@ -167,13 +135,17 @@ static int _run( const QString& mm,const QStringList& args,secrets& ss )
 
 		for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
-			const auto& a = it.volumePath() ;
-			const auto& b = it.mountPoint() ;
-			const auto& c = it.fileSystem() ;
+			const auto& m = it.vInfo ;
+
+			const auto& a = m.cipherPath() ;
+			const auto& b = m.mountPoint();
+			const auto& c = m.fileSystem() ;
 
 			if( a == volume || b == volume ){
 
-				if( siritask::encryptedFolderUnMount( { a,b,c,5 } ).success() ){
+				const auto& engine = engines::instance().getByName( c ) ;
+
+				if( siritask::encryptedFolderUnMount( { a,b,engine,5 } ).success() ){
 
 					siritask::deleteMountFolder( b ) ;
 
@@ -191,7 +163,7 @@ static int _run( const QString& mm,const QStringList& args,secrets& ss )
 
 		for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
-			it.printVolumeInfo() ;
+			std::cout << it.vInfo.toString().toStdString() << std::endl ;
 		}
 
 		return 0 ;
@@ -317,12 +289,31 @@ void sirikali::closeApplication( int s,const QString& e )
 {
 	m_secrets.close() ;
 
-	if( utility::platformIsWindows() && m_ui && !m_emergencyShuttingDown ){
+	if( m_ui && !m_emergencyShuttingDown ){
 
-		if( m_ui->tableWidget->rowCount() > 0 ){
+		auto m = processManager::get().enginesList() ;
 
-			auto m = tr( "Close All Volumes Before Quitting The Application" ) ;
-			return DialogMsg( this ).ShowUIOK( tr( "WARNING" ),m ) ;
+		if( utility::platformIsWindows() ){
+
+			if( m.size() > 0 ){
+
+				auto m = tr( "Close All Volumes Before Quitting The Application" ) ;
+				return DialogMsg( this ).ShowUIOK( tr( "WARNING" ),m ) ;
+			}
+		}else{
+			if( m.size() > 0 ){
+
+				QString s = m[ 0 ]->name() ;
+
+				for( size_t i = 1 ; i < m.size() ; i++ ){
+
+					s += "," + m[ i ]->name() ;
+				}
+
+				auto m = tr( "Close The Following File System(s) Before Quitting The Application\n\"%1\"" ) ;
+
+				return DialogMsg( this ).ShowUIOK( tr( "WARNING" ),m.arg( s ) ) ;
+			}
 		}
 	}
 
@@ -489,9 +480,7 @@ void sirikali::setUpApp( const QString& volume )
 
 	this->showTrayIcon() ;
 
-	this->startGUI( volume ) ;
-
-	if( utility::platformIsNOTWindows() ){
+	if( utility::platformIsNOTWindows() && checkUpdates::hasNetworkSupport() ){
 
 		QTimer::singleShot( settings::instance().checkForUpdateInterval(),this,SLOT( autoUpdateCheck() ) ) ;
 	}
@@ -521,6 +510,8 @@ void sirikali::setUpApp( const QString& volume )
 	}
 
 	this->updateFavoritesInContextMenu() ;
+
+	this->startGUI( volume ) ;
 }
 
 void sirikali::showTrayIcon()
@@ -618,8 +609,12 @@ void sirikali::setUpAppMenu()
 
 	if( utility::platformIsNOTWindows() ){
 
-		m->addAction( _addAction( false,false,tr( "Check For Updates" ),"Check For Updates",
-					  SLOT( updateCheck() ) ) ) ;
+		auto ac = _addAction( false,false,tr( "Check For Updates" ),"Check For Updates",
+				      SLOT( updateCheck() ) ) ;
+
+		ac->setEnabled( checkUpdates::hasNetworkSupport() ) ;
+
+		m->addAction( ac ) ;
 	}
 	m->addAction( _addAction( false,false,tr( "Unmount All" ),
 				  "Unmount All",SLOT( unMountAll() ) ) ) ;
@@ -697,6 +692,53 @@ void sirikali::favoriteClicked()
 	} ) ;
 }
 
+void sirikali::mountAll()
+{
+	mountinfo::unlockedVolumes().then( [ this ]( const mountinfo::List& m ){
+
+		this->updateVolumeList( m,false ) ;
+
+		this->autoUnlockVolumes( m,true ) ;
+	} ) ;
+}
+
+void sirikali::mountFavorite( const QString& e )
+{
+	auto _found = [ & ]( const favorites::volEntry& e,
+			     const QString& mountPointPath,
+			     const QString& volumePath ){
+
+		const auto& s = e.favorite() ;
+
+		if( mountPointPath.isEmpty() ){
+
+			return s.volumePath == volumePath ;
+		}else{
+			return s.volumePath == volumePath && s.mountPointPath == mountPointPath ;
+		}
+	} ;
+
+	QString volumePath ;
+	QString mountPointPath ;
+
+	utility2::stringListToStrings( utility::split( e ),volumePath,mountPointPath ) ;
+
+	for( auto&& it : favorites::instance().readVolumeList() ){
+
+		if( _found( it,mountPointPath,volumePath ) ){
+
+			favorites::volumeList s ;
+			s.emplace_back( std::move( it ) ) ;
+
+			auto mm = this->autoUnlockVolumes( std::move( s ),true ) ;
+
+			this->mountMultipleVolumes( std::move( mm ) ) ;
+
+			break ;
+		}
+	}
+}
+
 void sirikali::favoriteClicked( QAction * ac )
 {
 	auto e = ac->objectName() ;
@@ -714,65 +756,9 @@ void sirikali::favoriteClicked( QAction * ac )
 	}else{
 		if( e == "Mount All" ){
 
-			auto v = favorites::instance().readVolumeList() ;
-
-			auto e = settings::instance().autoMountBackEnd() ;
-
-			if( e.isValid() ){
-
-				auto m = m_secrets.walletBk( e.bk() ) ;
-
-				m.open( [ & ](){
-
-					for( auto& it : v ){
-
-						auto s = m->readValue( it.favorite().volumePath ) ;
-
-						if( !s.isEmpty() ){
-
-							it.setPassword( std::move( s ) ) ;
-						}
-					}
-
-					return true ;
-				} ) ;
-			}
-
-			this->mountMultipleVolumes( _convert_lists( std::move( v ),false,false ) ) ;
+			this->mountAll() ;
 		}else{
-			auto _found = [ & ]( const favorites::volEntry& e,
-					     const QString& mountPointPath,
-					     const QString& volumePath ){
-
-				const auto& s = e.favorite() ;
-
-				if( mountPointPath.isEmpty() ){
-
-					return s.volumePath == volumePath ;
-				}else{
-					return s.volumePath == volumePath && s.mountPointPath == mountPointPath ;
-				}
-			} ;
-
-			QString volumePath ;
-			QString mountPointPath ;
-
-			utility2::stringListToStrings( utility::split( e ),volumePath,mountPointPath ) ;
-
-			for( auto&& it : favorites::instance().readVolumeList() ){
-
-				if( _found( it,mountPointPath,volumePath ) ){
-
-					favorites::volumeList s ;
-					s.emplace_back( std::move( it ) ) ;
-
-					auto mm = this->autoUnlockVolumes( std::move( s ),true ) ;
-
-					this->mountMultipleVolumes( std::move( mm ) ) ;
-
-					break ;
-				}
-			}
+			this->mountFavorite( e ) ;
 		}
 	}
 
@@ -791,23 +777,40 @@ void sirikali::setLocalizationLanguage( bool translate )
 
 void sirikali::startGUI( const QString& volume )
 {
-	auto m = mountinfo::unlockedVolumes().await() ;
+	auto m = settings::instance().autoMountFavoritesOnStartUp() ;
 
-	this->updateVolumeList( m ) ;
+	if( m_startHidden ){
 
-	if( !m_startHidden ){
+		this->startGUI( volume,m ) ;
+	}else{
+		this->disableAll() ;
 
 		this->showMainWindow() ;
+
+		auto s = settings::instance().delayBeforeAutoMountAtStartup() ;
+
+		utility::Timer( s * 1000,[ this,volume,m ](){
+
+			this->startGUI( volume,m ) ;
+		} ) ;
 	}
+}
 
-	if( settings::instance().autoMountFavoritesOnStartUp() ){
+void sirikali::startGUI( const QString& volume,bool autoMountAtStartUp )
+{
+	mountinfo::unlockedVolumes().then( [ = ]( mountinfo::List m ){
 
-		this->autoUnlockVolumes( m ) ;
-	}
+		this->updateVolumeList( m,false ) ;
 
-	this->autoMount( volume ) ;
+		if( autoMountAtStartUp ){
 
-	utility::applicationStarted() ;
+			this->autoUnlockVolumes( m ) ;
+		}
+
+		this->autoMount( volume ) ;
+
+		utility::applicationStarted() ;
+	} ) ;
 }
 
 void sirikali::createbackendwindow()
@@ -866,122 +869,145 @@ void sirikali::start()
 	oneinstance::instance( this,s.socketFullPath,x,std::move( cb ) ) ;
 }
 
+struct unlockVolumeStruct{
+	QString vol ;
+	QString volume ;
+	QString mountPath ;
+	QString backEnd ;
+	QString idleTime ;
+	QString cPath ;
+	QString keyFile ;
+	QString mOpt ;
+	bool reverse ;
+	bool mode ;
+};
+
+static int _unlockVolume( const unlockVolumeStruct& s,const QByteArray& key ){
+
+	auto m = [ & ](){
+
+		if( s.mountPath.isEmpty() ){
+
+			auto e = utility::mountPathPostFix( s.volume.split( "/" ).last() ) ;
+
+			return settings::instance().mountPath( e ) ;
+		}else{
+			return s.mountPath ;
+		}
+	}() ;
+
+	engines::engine::booleanOptions mmm ;
+
+	mmm.unlockInReverseMode = s.reverse ;
+	mmm.unlockInReadOnly    = s.mode ;
+
+	QString q ;
+
+	engines::engine::mountGUIOptions::mountOptions mm( s.idleTime,s.cPath,s.mOpt,q,q,q,mmm ) ;
+
+	engines::engine::cmdArgsList ss{ s.volume,m,key,mm } ;
+
+	siritask::mount sss{ ss,false,{ ss.cipherFolder,ss.configFilePath } } ;
+
+	if( utility::platformIsWindows() || sss.engine->runsInForeGround() ){
+
+		QString a = "This Action Is Not Supported On Windows Or With" ;
+		QString b = " Backends That Do Not Run In Background" ;
+
+		return _print_err( 1,a + b ) ;
+	}
+
+	auto e = siritask::encryptedFolderMount( sss ) ;
+
+	if( e == engines::engine::status::success ){
+
+		return 0 ;
+	}else{
+		return _print_err( 1,e.toString() ) ;
+	}
+}
+
+static secrets::wallet::walletKey _walletKey( const unlockVolumeStruct& s,secrets& secrets )
+{
+	using wxt = LXQt::Wallet::BackEnd ;
+
+	auto _supported = [ & ]( wxt e,const char * m ){
+
+		return s.backEnd == m && LXQt::Wallet::backEndIsSupported( e ) ;
+	} ;
+
+	if( _supported( wxt::internal,"internal" ) ){
+
+		return secrets.walletBk( wxt::internal ).getKey( s.volume ) ;
+
+	}else if( _supported( wxt::libsecret,"gnomewallet" ) ||
+		  _supported( wxt::libsecret,"libsecret" ) ){
+
+		return secrets.walletBk( wxt::libsecret ).getKey( s.volume ) ;
+
+	}else if( _supported( wxt::kwallet,"kwallet" ) ){
+
+		return secrets.walletBk( wxt::kwallet ).getKey( s.volume ) ;
+
+	}else if( _supported( wxt::osxkeychain,"osxkeychain" ) ){
+
+		return secrets.walletBk( wxt::osxkeychain ).getKey( s.volume ) ;
+
+	}else if( _supported( wxt::windows_dpapi,"windows_dpapi" ) ){
+
+		return secrets.walletBk( LXQt::Wallet::BackEnd::windows_dpapi ).getKey( s.volume ) ;
+	}else{
+		return secrets::wallet::walletKey{ false,true,"" } ;
+	}
+}
+
 int sirikali::unlockVolume( const QStringList& l,secrets& secrets )
 {
-	auto vol       = utility::cmdArgumentValue( l,"-d" ) ;
-	auto volume    = QDir( vol ).canonicalPath() ;
-	auto mountPath = utility::cmdArgumentValue( l,"-z" ) ;
-	auto backEnd   = utility::cmdArgumentValue( l,"-b" ) ;
-	auto mode      = utility::cmdArgumentValue( l,"-k","rw" ) == "ro" ;
-	auto idleTime  = utility::cmdArgumentValue( l,"-i" ) ;
-	auto cPath     = utility::cmdArgumentValue( l,"-c" ) ;
-	auto keyFile   = utility::cmdArgumentValue( l,"-f" ) ;
-	auto mOpt      = utility::cmdArgumentValue( l,"-o" ) ;
-	auto reverse   = l.contains( "-r" ) ;
+	unlockVolumeStruct s ;
 
-	if( vol.isEmpty() ){
+	s.vol       = utility::cmdArgumentValue( l,"-d" ) ;
+	s.volume    = QDir( s.vol ).canonicalPath() ;
+	s.mountPath = utility::cmdArgumentValue( l,"-z" ) ;
+	s.backEnd   = utility::cmdArgumentValue( l,"-b" ) ;
+	s.mode      = utility::cmdArgumentValue( l,"-k","rw" ) == "ro" ;
+	s.idleTime  = utility::cmdArgumentValue( l,"-i" ) ;
+	s.cPath     = utility::cmdArgumentValue( l,"-c" ) ;
+	s.keyFile   = utility::cmdArgumentValue( l,"-f" ) ;
+	s.mOpt      = utility::cmdArgumentValue( l,"-o" ) ;
+	s.reverse   = l.contains( "-r" ) ;
+
+	if( s.vol.isEmpty() ){
 
 		return _print_err( 1,tr( "ERROR: Volume Path Not Given." ) ) ;
 	}else{
-		auto _unlockVolume = [ & ]( const QByteArray& key ){
+		if( s.backEnd == "stdin" ){
 
-			auto m = [ & ](){
-
-				if( mountPath.isEmpty() ){
-
-					auto e = utility::mountPathPostFix( volume.split( "/" ).last() ) ;
-
-					return settings::instance().mountPath( e ) ;
-				}else{
-					return mountPath ;
-				}
-			}() ;
-
-			engines::engine::booleanOptions mmm ;
-
-			mmm.unlockInReverseMode = reverse ;
-			mmm.unlockInReadOnly    = mode ;
-
-			QString empty ;
-
-			engines::engine::mountGUIOptions::mountOptions mm( idleTime,
-									   cPath,
-									   mOpt,
-									   empty,
-									   empty,
-									   empty,
-									   mmm ) ;
-
-			auto e = siritask::encryptedFolderMount( { volume,m,key,mm } ) ;
-
-			if( e == engines::engine::status::success ){
-
-				return 0 ;
-			}else{
-				return _print_err( 1,e.toString() ) ;
-			}
-		} ;
-
-		if( backEnd == "stdin" ){
-
-			return _unlockVolume( [ & ](){
+			return _unlockVolume( s,[ & ](){
 
 				auto e = utility::readPassword() ;
 
-				if( keyFile.isEmpty() ){
+				if( s.keyFile.isEmpty() ){
 
 					return e ;
 				}else{
-					return crypto::hmac_key( keyFile,e ) ;
+					return crypto::hmac_key( s.keyFile,e ) ;
 				}
 			}() ) ;
 		}
 
-		if( backEnd == "keyfile" ){
+		if( s.backEnd == "keyfile" ){
 
-			auto key = utility::fileContents( keyFile ) ;
+			auto key = utility::fileContents( s.keyFile ) ;
 
 			if( utility::containsAtleastOne( key,'\n','\0','\r' ) ){
 
 				_print_err( 1,keyDialog::keyFileError() ) ;
 			}
 
-			return _unlockVolume( key ) ;
+			return _unlockVolume( s,key ) ;
 		}
 
-		auto w = [ & ](){
-
-			using wxt = LXQt::Wallet::BackEnd ;
-
-			auto _supported = [ & ]( wxt e,const char * s ){
-
-				return backEnd == s && LXQt::Wallet::backEndIsSupported( e ) ;
-			} ;
-
-			if( _supported( wxt::internal,"internal" ) ){
-
-				return secrets.walletBk( wxt::internal ).getKey( volume ) ;
-
-			}else if( _supported( wxt::libsecret,"gnomewallet" ) ||
-				  _supported( wxt::libsecret,"libsecret" ) ){
-
-				return secrets.walletBk( wxt::libsecret ).getKey( volume ) ;
-
-			}else if( _supported( wxt::kwallet,"kwallet" ) ){
-
-				return secrets.walletBk( wxt::kwallet ).getKey( volume ) ;
-
-			}else if( _supported( wxt::osxkeychain,"osxkeychain" ) ){
-
-				return secrets.walletBk( wxt::osxkeychain ).getKey( volume ) ;
-
-			}else if( _supported( wxt::windows_dpapi,"windows_dpapi" ) ){
-
-				return secrets.walletBk( LXQt::Wallet::BackEnd::windows_dpapi ).getKey( volume ) ;
-			}else{
-				return secrets::wallet::walletKey{ false,true,"" } ;
-			}
-		}() ;
+		auto w = _walletKey( s,secrets ) ;
 
 		if( w.opened ){
 
@@ -989,7 +1015,7 @@ int sirikali::unlockVolume( const QStringList& l,secrets& secrets )
 
 				return _print_err( 1,tr( "ERROR: Key Not Found In The Backend." ) ) ;
 			}else{
-				return _unlockVolume( w.key.toUtf8() ) ;
+				return _unlockVolume( s,w.key.toUtf8() ) ;
 			}
 		}else{
 			return _print_err( 1,tr( "ERROR: Failed To Unlock Requested Backend." ) ) ;
@@ -1001,14 +1027,14 @@ void sirikali::mountMultipleVolumes( keyDialog::volumeList e )
 {
 	if( !e.empty() ){
 
-		m_disableEnableAll = true ;
+		m_allowEnableAll.setFalse() ;
 
 		this->disableAll() ;
 
 		auto done = [ this ](){
 
 			this->setAcceptDrops( true ) ;
-			m_disableEnableAll = false ;
+			m_allowEnableAll.setTrue() ;
 			this->enableAll() ;
 		} ;
 
@@ -1021,6 +1047,9 @@ void sirikali::mountMultipleVolumes( keyDialog::volumeList e )
 				     std::move( e ),
 				     std::move( done ),
 				     [ this ](){ this->updateList() ; } ) ;
+	}else{
+		m_allowEnableAll.setTrue() ;
+		this->enableAll() ;
 	}
 }
 
@@ -1044,34 +1073,51 @@ void sirikali::autoMountFavoritesOnAvailable( QString m )
 	}
 }
 
-void sirikali::autoUnlockVolumes( const std::vector< volumeInfo >& s )
+void sirikali::autoUnlockVolumes( const mountinfo::List& s,bool autoSetAutoMount )
 {
 	favorites::volumeList e ;
 
-	auto _mounted = [ & ]( const QString& e ){
+	auto _not_mounted = [ & ]( const favorites::entry& e ){
 
 		for( const auto& it : s ){
 
-			if( it.volumePath() == e ){
+			const auto& m = it.vInfo ;
 
-				return true ;
+			if( it.engine.setsCipherPath() ){
+
+				if( m.cipherPath() == e.volumePath ){
+
+					return false ;
+				}
+
+			}else if( m.mountPoint() == e.mountPointPath ){
+
+				return false ;
 			}
 		}
 
-		return false ;
+		return true ;
 	} ;
 
 	for( auto&& it : favorites::instance().readVolumeList() ){
 
 		const auto& m = it.favorite() ;
 
-		if( m.autoMount && !_mounted( m.volumePath )){
+		if( autoSetAutoMount ){
 
-			e.emplace_back( std::move( it ) ) ;
+			if( _not_mounted( m ) ){
+
+				e.emplace_back( std::move( it ) ) ;
+			}
+		}else{
+			if( m.autoMount && _not_mounted( m ) ){
+
+				e.emplace_back( std::move( it ) ) ;
+			}
 		}
 	}
 
-	this->mountMultipleVolumes( this->autoUnlockVolumes( std::move( e ) ) ) ;
+	this->mountMultipleVolumes( this->autoUnlockVolumes( std::move( e ),false,false,autoSetAutoMount ) ) ;
 }
 
 void sirikali::autoMount( keyDialog::volumeList& q,
@@ -1133,7 +1179,7 @@ keyDialog::volumeList sirikali::autoMount( keyDialog::volumeList l,bool autoOpen
 
 		bool u = it.engine->known() && it.engine->requiresNoPassword() ;
 
-		if(  u || m.volumeNeedNoPassword || !m.password.isEmpty() ){
+		if( u || m.volumeNeedNoPassword || !m.password.isEmpty() ){
 
 			this->autoMount( e,std::move( it ),s,autoOpenFolderOnMount ) ;
 		}else{
@@ -1144,7 +1190,7 @@ keyDialog::volumeList sirikali::autoMount( keyDialog::volumeList l,bool autoOpen
 	return e ;
 }
 
-QByteArray static _get_key( const keyDialog::entry& it,secrets::wallet& m ){
+QByteArray static _get_key( QString& debug,const keyDialog::entry& it,secrets::wallet& m ){
 
 	const auto& volumePath = it.volEntry.favorite().volumePath ;
 
@@ -1152,12 +1198,16 @@ QByteArray static _get_key( const keyDialog::entry& it,secrets::wallet& m ){
 
 	if( volumePath.startsWith( name + " ",Qt::CaseInsensitive ) ){
 
+		debug += "\n3. Trying to get password from wallet for path: " + volumePath ;
+
 		auto ee = m->readValue( volumePath ) ;
 
 		if( ee.isEmpty() ){
 
 			auto ss = name.toLower() + " " + it.engine.cipherFolder() ;
 
+			debug += "\n3.1 Retrying to get password from wallet for path: " + ss ;
+
 			ee = m->readValue( ss ) ;
 		}
 
@@ -1169,11 +1219,12 @@ QByteArray static _get_key( const keyDialog::entry& it,secrets::wallet& m ){
 
 keyDialog::volumeList sirikali::autoUnlockVolumes( favorites::volumeList ss,
 						   bool autoOpenFolderOnMount,
-						   bool skipUnknown )
+						   bool skipUnknown,
+						   bool autoSetAutoMount )
 {
 	const auto& mm = utility::miscOptions::instance() ;
 
-	auto l = _convert_lists( std::move( ss ),mm.starting(),skipUnknown ) ;
+	auto l = keyDialog::fromFavoritesList( std::move( ss ),mm.starting(),skipUnknown ) ;
 
 	if( l.empty() ){
 
@@ -1182,39 +1233,112 @@ keyDialog::volumeList sirikali::autoUnlockVolumes( favorites::volumeList ss,
 
 	auto ee = settings::instance().autoMountBackEnd() ;
 
-	if( ee.isInvalid() ){
+	auto m = [ & ](){
 
-		utility::debug() << "Warning, Wallet Not Set" ;
-		return this->autoMount( std::move( l ),autoOpenFolderOnMount ) ;
-	}
+		if( ee.isInvalid() ){
 
-	auto m = m_secrets.walletBk( ee.bk() ) ;
+			utility::debug() << "Warning, Wallet Not Set" ;
 
-	if( !m ){
+			return secrets::wallet() ;
+		}else{
+			auto m = m_secrets.walletBk( ee.bk() ) ;
 
-		utility::debug() << "Warning, Failed To Set Selected Wallet" ;
-		return this->autoMount( std::move( l ),autoOpenFolderOnMount ) ;
-	}
+			if( !m ){
 
-	if( !m.open() ){
+				utility::debug() << "Warning, Failed To Set Selected Wallet" ;
+				return secrets::wallet() ;
+			}
 
-		utility::debug() << "Warning, Failed To Open Selected Wallet" ;
-		return this->autoMount( std::move( l ),autoOpenFolderOnMount ) ;
-	}
+			if( m.open() ){
+
+				return m ;
+			}else{
+				utility::debug() << "Warning, Failed To Open Selected Wallet" ;
+				return secrets::wallet() ;
+			}
+		}
+	}() ;
 
 	keyDialog::volumeList e ;
 
 	auto s = settings::instance().showMountDialogWhenAutoMounting() ;
 
+	auto _mountTooLong = [ & ]( QString& s,keyDialog::entry&& m ){
+
+		s += "\n6. Unconditionally showing mount dialog window" ;
+		s += " because the backend takes too long to unlock" ;
+
+		m.volEntry.setAutoMount( true ) ;
+
+		e.emplace_back( std::move( m ) ) ;
+	} ;
+
+	QString debug ;
+
+	m_allowEnableAll.setFalse() ;
+
+	auto disableAllRaii = utility2::make_raii( [ this ](){ m_allowEnableAll.setTrue() ; } ) ;
+
 	for( auto&& it : l ){
 
-		it.volEntry.setPassword( _get_key( it,m ) ) ;
+		debug = "1. Processing favorite entry: " + it.volEntry.favorite().volumePath ;
+
+		if( autoSetAutoMount ){
+
+			it.volEntry.setAutoMount( true ) ;
+		}
 
 		if( it.volEntry.password().isEmpty() ){
 
-			e.emplace_back( std::move( it ) ) ;
+			debug += "\n2. Favorite entry does not have a password" ;
+
+			if( m ){
+
+				/*
+				 * _get_key sets entry 3.
+				 */
+				it.volEntry.setPassword( _get_key( debug,it,m ) ) ;
+
+				if( it.volEntry.password().isEmpty() ){
+
+					debug += "\n4. Favorite entry does not exist in the wallet" ;
+				}else{
+					debug += "\n4. Password obtained from wallet" ;
+				}
+			}else{
+				debug += "\n3. Failed to read password from wallet" ;
+			}
 		}else{
-			this->autoMount( e,std::move( it ),s,autoOpenFolderOnMount ) ;
+			debug += "\n2. Favorite entry already has a password" ;
+		}
+
+		if( it.volEntry.password().isEmpty() ){
+
+			if( it.engine->requiresNoPassword() || it.volEntry.favorite().volumeNeedNoPassword ){
+
+				debug += "\n5. Engine requires no password or favorite need no password" ;
+
+				if( it.engine->takesTooLongToUnlock() ){
+
+					_mountTooLong( debug,std::move( it ) ) ;
+					utility::debug() << debug ;
+				}else{
+					utility::debug() << debug ;
+					this->autoMount( e,std::move( it ),s,autoOpenFolderOnMount ) ;
+				}
+			}else{
+				utility::debug() << debug ;
+				e.emplace_back( std::move( it ) ) ;
+			}
+		}else{
+			if( it.engine->takesTooLongToUnlock() ){
+
+				_mountTooLong( debug,std::move( it ) ) ;
+				utility::debug() << debug ;
+			}else{
+				utility::debug() << debug ;
+				this->autoMount( e,std::move( it ),s,autoOpenFolderOnMount ) ;
+			}
 		}
 	}
 
@@ -1252,13 +1376,15 @@ void sirikali::genericVolumeProperties()
 {
 	this->disableAll() ;
 
-	if( utility::platformIsWindows() ){
+	auto table = m_ui->tableWidget ;
 
-		auto table = m_ui->tableWidget ;
+	auto row = table->currentRow() ;
 
-		auto row = table->currentRow() ;
+	const auto& engine = engines::instance().getByName( table->item( row,2 )->text() ) ;
 
-		auto m = SiriKali::Windows::volumeProperties( table->item( row,1 )->text() ) ;
+	if( utility::platformIsWindows() || engine.runsInForeGround() ){
+
+		auto m = processManager::get().volumeProperties( table->item( row,1 )->text() ) ;
 
 		if( m.isEmpty() ){
 
@@ -1286,11 +1412,11 @@ void sirikali::genericVolumeProperties()
 
 	for( const auto& it : mountinfo::unlockedVolumes().await() ){
 
-		if( it.mountPoint() == s ){
+		if( it.vInfo.mountPoint() == s ){
 
 			DialogMsg( this ).ShowUIInfo( tr( "INFORMATION" ),true,[ & ](){
 
-				auto s = it.mountOptions() ;
+				auto s = it.vInfo.mountOptions() ;
 
 				s.replace( ",","\n\n" ) ;
 				s.replace( "ro\n\n","mode=read only\n\n" ) ;
@@ -1699,35 +1825,44 @@ void sirikali::createVolume( QAction * ac )
 	}
 }
 
-void sirikali::autoMount( const QString& volume )
+void sirikali::autoMount( const QString& vv )
 {
-	if( !volume.isEmpty() ){
+	auto volume = vv ;
 
-		auto& favorites = favorites::instance() ;
+	while( volume.endsWith( '/' ) ) {
 
-		auto s = settings::instance().autoOpenFolderOnMount() ;
-
-		auto m = [ & ](){
-
-			for( auto&& it : favorites.readVolumeList() ){
-
-				if( it.favorite().volumePath == volume ){
-
-					favorites::volumeList mm ;
-					mm.emplace_back( std::move( it ) ) ;
-
-					return this->autoUnlockVolumes( std::move( mm ),s ) ;
-				}
-			}
-
-			favorites::volumeList mm ;
-			mm.emplace_back( favorites::entry( volume ),true ) ;
-
-			return this->autoUnlockVolumes( std::move( mm ),s ) ;
-		}() ;
-
-		this->mountMultipleVolumes( std::move( m ) ) ;
+		volume = utility::removeLast( volume,1 ) ;
 	}
+
+	if( volume.isEmpty() ){
+
+		return this->enableAll() ;
+	}
+
+	auto& favorites = favorites::instance() ;
+
+	auto s = settings::instance().autoOpenFolderOnMount() ;
+
+	auto m = [ & ](){
+
+		for( auto&& it : favorites.readVolumeList() ){
+
+			if( it.favorite().volumePath == volume ){
+
+				favorites::volumeList mm ;
+				mm.emplace_back( std::move( it ) ) ;
+
+				return this->autoUnlockVolumes( std::move( mm ),s ) ;
+			}
+		}
+
+		favorites::volumeList mm ;
+		mm.emplace_back( favorites::entry( volume ),true ) ;
+
+		return this->autoUnlockVolumes( std::move( mm ),s ) ;
+	}() ;
+
+	this->mountMultipleVolumes( std::move( m ) ) ;
 }
 
 void sirikali::unlockVolume( bool dir )
@@ -1761,89 +1896,89 @@ QFont sirikali::getSystemVolumeFont()
 	return f ;
 }
 
-void sirikali::updateFavoritesInContextMenu()
-{
-	class mountedVolumes{
-	public:
-		mountedVolumes( QTableWidget * table ) :
-			m_cipherPath( tablewidget::columnEntries( table,0 ) ),
-			m_mountPath( tablewidget::columnEntries( table,1 ) )
-		{
-		}
-		int row( const QString& e )
-		{
-			auto a = utility::split( e,'\n' ) ;
+class mountedVolumes{
+public:
+	mountedVolumes( QTableWidget * table ) :
+		m_cipherPath( tablewidget::columnEntries( table,0 ) ),
+		m_mountPath( tablewidget::columnEntries( table,1 ) )
+	{
+	}
+	int row( const QString& e )
+	{
+		auto a = utility::split( e,'\n' ) ;
 
-			if( a.size() == 1 ){
+		if( a.size() == 1 ){
 
-				const auto& m = a.at( 0 ) ;
+			const auto& m = a.at( 0 ) ;
 
-				for( int i = 0 ; i < m_cipherPath.size() ; i++ ){
-
-					if( m == m_cipherPath.at( i ) ){
-
-						return i ;
-					}
-				}
-			}else{
-				const auto& x = a.at( 0 ) ;
-				const auto& y = a.at( 1 ) ;
-
-				for( int i = 0 ; i < m_cipherPath.size() ; i++ ){
-
-					if( x == m_cipherPath.at( i ) && y == m_mountPath.at( i ) ){
-
-						return i ;
-					}
-				}
-			}
-
-			return -1 ;
-		}
-
-		bool mounted( const QString& e )
-		{
-			auto s = this->row( e ) ;
-
-			if( s == -1 ){
-
-				return false ;
-			}else{
-				m_cipherPath.removeAt( s ) ;
-				m_mountPath.removeAt( s ) ;
-
-				return true ;
-			}
-		}
-
-		void addMounted( QMenu * m,bool e )
-		{
 			for( int i = 0 ; i < m_cipherPath.size() ; i++ ){
 
-				auto ac = new QAction( m ) ;
+				if( m == m_cipherPath.at( i ) ){
 
-				QString n ;
-
-				if( e ){
-
-					n = m_cipherPath.at( i ) + "\n" + m_mountPath.at( i ) ;
-				}else{
-					n = m_cipherPath.at( i ) ;
+					return i ;
 				}
+			}
+		}else{
+			const auto& x = a.at( 0 ) ;
+			const auto& y = a.at( 1 ) ;
 
-				ac->setText( n ) ;
-				ac->setObjectName( n ) ;
-				ac->setCheckable( true ) ;
-				ac->setChecked( true ) ;
+			for( int i = 0 ; i < m_cipherPath.size() ; i++ ){
 
-				m->addAction( ac ) ;
+				if( x == m_cipherPath.at( i ) && y == m_mountPath.at( i ) ){
+
+					return i ;
+				}
 			}
 		}
-	private:
-		QStringList m_cipherPath ;
-		QStringList m_mountPath ;
-	};
 
+		return -1 ;
+	}
+
+	bool mounted( const QString& e )
+	{
+		auto s = this->row( e ) ;
+
+		if( s == -1 ){
+
+			return false ;
+		}else{
+			m_cipherPath.removeAt( s ) ;
+			m_mountPath.removeAt( s ) ;
+
+			return true ;
+		}
+	}
+
+	void addMounted( QMenu * m,bool e )
+	{
+		for( int i = 0 ; i < m_cipherPath.size() ; i++ ){
+
+			auto ac = new QAction( m ) ;
+
+			QString n ;
+
+			if( e ){
+
+				n = m_cipherPath.at( i ) + "\n" + m_mountPath.at( i ) ;
+			}else{
+				n = m_cipherPath.at( i ) ;
+			}
+
+			ac->setText( n ) ;
+			ac->setObjectName( n ) ;
+			ac->setCheckable( true ) ;
+			ac->setChecked( true ) ;
+
+			m->addAction( ac ) ;
+		}
+	}
+private:
+	QStringList m_cipherPath ;
+	QStringList m_mountPath ;
+};
+
+void sirikali::updateFavoritesInContextMenu()
+{
 	if( !settings::instance().showFavoritesInContextMenu() ){
 
 		return ;
@@ -1899,7 +2034,7 @@ void sirikali::updateFavoritesInContextMenu()
 
 void sirikali::runIntervalCustomCommand( const QString& cmd )
 {
-	this->processMountedVolumes( [ = ]( const sirikali::mountedEntry& s ){
+	this->processMountedVolumes( m_ui->tableWidget,[ & ]( const sirikali::mountedEntry& s ){
 
 		QStringList args{ s.cipherPath,s.mountPoint,s.volumeType } ;
 
@@ -1951,7 +2086,7 @@ void sirikali::addEntryToTable( const QStringList& l )
 
 void sirikali::addEntryToTable( const volumeInfo& e )
 {
-	this->addEntryToTable( e.mountInfo().minimalList() ) ;
+	this->addEntryToTable( e.toStringList() ) ;
 }
 
 void sirikali::removeEntryFromTable( QString volume )
@@ -1969,20 +2104,20 @@ void sirikali::removeEntryFromTable( QString volume )
 	}
 }
 
-void sirikali::updateList( const volumeInfo& entry )
+void sirikali::updateList( const volumeInfo& e )
 {
-	if( entry.isValid() ){
+	if( e.isValid() ){
 
 		auto table = m_ui->tableWidget ;
 
-		auto row = tablewidget::columnHasEntry( table,entry.mountPoint(),1 ) ;
+		auto row = tablewidget::columnHasEntry( table,e.mountPoint(),1 ) ;
 
 		if( row == -1 ){
 
 			row = tablewidget::addRow( table ) ;
 		}
 
-		tablewidget::updateRow( table,entry.mountInfo().minimalList(),row,this->font() ) ;
+		tablewidget::updateRow( table,e.toStringList(),row,this->font() ) ;
 
 		tablewidget::selectRow( table,row ) ;
 	}
@@ -1990,13 +2125,28 @@ void sirikali::updateList( const volumeInfo& entry )
 
 engines::engine::cmdStatus sirikali::unMountVolume( const sirikali::mountedEntry& e )
 {
-	auto s = siritask::encryptedFolderUnMount( { e.cipherPath,e.mountPoint,e.volumeType,5 } ) ;
+	const auto& engine = engines::instance().getByName( e.volumeType ) ;
 
-	const auto& m = s.engine() ;
+	auto mm = utility2::make_raii( [ this ](){
 
-	if( s.success() && m.backendRequireMountPath() ){
+		m_mountInfo.announceEvents( true ) ;
+		m_allowEnableAll.setTrue() ;
+	} ) ;
 
-		siritask::deleteMountFolder( e.mountPoint ) ;
+	if( utility::platformIsLinux() && engine.runsInForeGround() ){
+
+		m_mountInfo.announceEvents( false ) ;
+		m_allowEnableAll.setFalse() ;
+	}
+
+	auto s = siritask::encryptedFolderUnMount( { e.cipherPath,e.mountPoint,engine,5 } ) ;
+
+	if( s.success() && engine.backendRequireMountPath() ){
+
+		utility::Timer( 1000,[ s = e.mountPoint ](){
+
+			siritask::deleteMountFolder( s ) ;
+		} ) ;
 	}
 
 	return s ;
@@ -2041,24 +2191,6 @@ void sirikali::pbUmount()
 	}
 }
 
-void sirikali::processMountedVolumes( std::function< void( const sirikali::mountedEntry& ) > function )
-{
-	auto table = m_ui->tableWidget ;
-
-	const auto cipherFolders = tablewidget::columnEntries( table,0 ) ;
-	const auto mountPoints   = tablewidget::columnEntries( table,1 ) ;
-	const auto fileSystems   = tablewidget::columnEntries( table,2 ) ;
-
-	for( auto r = cipherFolders.size() - 1 ; r >= 0 ; r-- ){
-
-		const auto& a = cipherFolders.at( r ) ;
-		const auto& b = mountPoints.at( r ) ;
-		const auto& c = fileSystems.at( r ) ;
-
-		function( { a,b,c } ) ;
-	}
-}
-
 void sirikali::emergencyShutDown()
 {
 	m_emergencyShuttingDown = true ;
@@ -2067,7 +2199,7 @@ void sirikali::emergencyShutDown()
 
 	m_mountInfo.announceEvents( false ) ;
 
-	this->processMountedVolumes( [ this ]( const sirikali::mountedEntry& e ){
+	this->processMountedVolumes( m_ui->tableWidget,[ this ]( const sirikali::mountedEntry& e ){
 
 		this->unMountVolume( e ) ;
 	} ) ;
@@ -2078,12 +2210,13 @@ void sirikali::emergencyShutDown()
 void sirikali::unMountAll()
 {
 	m_mountInfo.announceEvents( false ) ;
+	m_allowEnableAll.setFalse( 2020 ) ;
 
 	this->disableAll() ;
 
 	utility::waitForOneSecond() ;
 
-	this->processMountedVolumes( [ this ]( const mountedEntry& e ){
+	this->processMountedVolumes( m_ui->tableWidget,[ this ]( const mountedEntry& e ){
 
 		if( this->unMountVolume( e ).success() ){
 
@@ -2105,6 +2238,8 @@ void sirikali::unMountAll()
 		}
 	}
 
+	m_allowEnableAll.setTrue( 2020 ) ;
+
 	this->updateList() ;
 
 	this->enableAll() ;
@@ -2141,16 +2276,19 @@ void sirikali::pbUpdate()
 	this->updateList() ;
 }
 
-void sirikali::updateVolumeList( const std::vector< volumeInfo >& r )
+void sirikali::updateVolumeList( const mountinfo::List& r,bool enableAll )
 {
 	tablewidget::clearTable( m_ui->tableWidget ) ;
 
 	for( const auto& it : r ){
 
-		this->updateList( it ) ;
+		this->updateList( it.vInfo ) ;
 	}
 
-	this->enableAll() ;
+	if( enableAll ){
+
+		this->enableAll() ;
+	}
 }
 
 void sirikali::slotCurrentItemChanged( QTableWidgetItem * current,QTableWidgetItem * previous )
@@ -2160,20 +2298,21 @@ void sirikali::slotCurrentItemChanged( QTableWidgetItem * current,QTableWidgetIt
 
 void sirikali::disableAll()
 {
-	if( !utility::platformIsOSX() ){
+	m_ui->pbmenu->setEnabled( false ) ;
+	m_ui->tableWidget->setEnabled( false ) ;
+	m_ui->pbunlockvolume->setEnabled( false ) ;
+	m_ui->pbcreate->setEnabled( false ) ;
+	m_ui->pbFavorites->setEnabled( false ) ;
+
+	if( !utility::miscOptions::instance().starting() ){
 
-		m_ui->pbmenu->setEnabled( false ) ;
 		m_ui->pbupdate->setEnabled( false ) ;
-		m_ui->tableWidget->setEnabled( false ) ;
-		m_ui->pbunlockvolume->setEnabled( false ) ;
-		m_ui->pbcreate->setEnabled( false ) ;
-		m_ui->pbFavorites->setEnabled( false ) ;
 	}
 }
 
 void sirikali::enableAll()
 {
-	if( !m_disableEnableAll ){
+	if( m_allowEnableAll ){
 
 		m_ui->pbmenu->setEnabled( true ) ;
 		m_ui->pbupdate->setEnabled( true ) ;
