@@ -42,6 +42,7 @@
 #include "settings.h"
 #include "systemsignalhandler.h"
 #include "keydialog.h"
+#include "tablewidget.h"
 
 #include <vector>
 
@@ -95,6 +96,7 @@ private slots:
 	void closeApplication( int = 0,const QString& = QString() ) ;
 	void unlockVolume( bool ) ;
 	void startGUI( const QString& ) ;
+	void startGUI( const QString&,bool ) ;
 	void autoMount( const QString& ) ;
 	void defaultButton( void ) ;
 	void itemClicked( QTableWidgetItem * ) ;
@@ -123,6 +125,9 @@ private slots:
 	void updateCheck( void ) ;
 	void autoMountFavoritesOnAvailable( QString ) ;
 private:
+	void mountAll() ;
+	void mountFavorite( const QString& ) ;
+
 	configOptions::functions configOption() ;
 
 	void showTrayIcon() ;
@@ -142,7 +147,7 @@ private:
 
 	void updateFavoritesInContextMenu( void ) ;
 	void runIntervalCustomCommand( const QString& ) ;
-	void updateVolumeList( const std::vector< volumeInfo >& ) ;
+	void updateVolumeList( const mountinfo::List&,bool enableAll = true ) ;
 	void openMountPoint( const QString& ) ;
 	void setLocalizationLanguage( bool ) ;
 	void dragEnterEvent( QDragEnterEvent * ) ;
@@ -156,10 +161,11 @@ private:
 	void setUpShortCuts( void ) ;
 	void showMainWindow( void ) ;
 	void raiseWindow( const QString& = QString() ) ;
-	void autoUnlockVolumes( const std::vector< volumeInfo >& ) ;	
+	void autoUnlockVolumes( const mountinfo::List&,bool autoSetAutoMount = false ) ;
 	keyDialog::volumeList autoUnlockVolumes( favorites::volumeList,
 						 bool autoOpenFolderOnMount = false,
-						 bool skipUnknown = false ) ;
+						 bool skipUnknown = false,
+						 bool autoSetAutoMount = false ) ;
 
 	struct mountedEntry{
 		const QString& cipherPath ;
@@ -167,7 +173,22 @@ private:
 		const QString& volumeType ;
 	};
 
-	void processMountedVolumes( std::function< void( const sirikali::mountedEntry& ) > function ) ;
+	template< typename Function >
+	void processMountedVolumes( QTableWidget * table,Function function )
+	{
+		const auto cipherFolders = tablewidget::columnEntries( table,0 ) ;
+		const auto mountPoints   = tablewidget::columnEntries( table,1 ) ;
+		const auto fileSystems   = tablewidget::columnEntries( table,2 ) ;
+
+		for( auto r = cipherFolders.size() - 1 ; r >= 0 ; r-- ){
+
+			const auto& a = cipherFolders.at( r ) ;
+			const auto& b = mountPoints.at( r ) ;
+			const auto& c = fileSystems.at( r ) ;
+
+			function( { a,b,c } ) ;
+		}
+	}
 
 	engines::engine::cmdStatus unMountVolume( const sirikali::mountedEntry& ) ;
 
@@ -195,7 +216,6 @@ private:
 
 	bool m_startHidden ;
 	bool m_autoOpenFolderOnMount ;
-	bool m_disableEnableAll = false ;
 	bool m_emergencyShuttingDown = false ;
 
 	QString m_sharedFolderPath ;
@@ -214,6 +234,47 @@ private:
 	systemSignalHandler m_signalHandler ;
 
 	const QStringList m_argumentList ;
+
+	class allowEnableAll{
+
+	public:
+		operator bool() const
+		{
+			return m_allowEnableAll ;
+		}
+		void setTrue( int token = -1 )
+		{
+			/*
+			 * Only the person who knows the token can uset the option.
+			 * Normal users set token of -1, super users set higher number tokens.
+			 */
+			if( token == m_token ){
+
+				m_token = -1 ;
+				m_allowEnableAll = true ;
+			}
+		}
+		void setFalse( int token = -1 )
+		{
+			if( token != -1 && m_token != -1 ){
+
+				utility::debug() << "Warning: Two users with super token just collided" ;
+
+				return ;
+			}
+			/*
+			 * Option can be changed only if token is unset or set by normal user.
+			 */
+			if( m_token == -1 ){
+
+				m_token = token ;
+				m_allowEnableAll = false ;
+			}
+		}
+	private:
+		int m_token = -1 ;
+		bool m_allowEnableAll = true ;
+	} m_allowEnableAll ;
 };
 
 #endif // MAINWINDOW_H
